//  Copyright (C) University of Vienna, 2013-2015
//  All rights reserved.
//
//  Author: Ali Baharev
//
//  Please refer to the LICENSE.txt file in the top directory
//
package sdms;

import java.util.Arrays;

import org.apache.logging.log4j.*;

import tools.math.GlobalMathConstants;

public final class Main {
  
    static {

        ClassLoader.getSystemClassLoader().setDefaultAssertionStatus(true);
        
        String propertyName = "log4j.configurationFile";
        
        if (System.getProperty(propertyName)==null) {
            
            System.setProperty(propertyName, "config/log4j2.xml");
        }
        
        tools.math.GlobalMathConstants.setNetlibImpl(GlobalMathConstants.TYPE_NETLIB_JAVA);
    }

    public static void main(String[] args) {
        
        Util.testAssertationsEnabled();
        
        Util.testLoggerEnabled();
        
        Logger log = LogManager.getLogger();
        
        log.info("Running with arguments: "+Arrays.toString(args));
        
        if (args.length < 1) {
            log.error("Give at least one file name as command line argument!");
            return;
        }
        
       try {
 
        Options options = new Options("config" + java.io.File.separator + "algorithm.cfg");

        for (String file : args) {
            
            log.info("Processing test problem "+file);
            
            long start = System.nanoTime();
            
            new Algorithm().run(options, file);
            
            long end = System.nanoTime();
            
            log.info("Processing took "+Util.toTimeStamp(end-start));
        }

        }
        catch (NullPointerException | IndexOutOfBoundsException | AssertionError e) {
            e.printStackTrace();
            log.fatal("Ooops, this is a bug, exiting...");
            return;
        }
        catch (ProgramTerminationRequested e) {
            e.printStackTrace();
            log.fatal("Fatal error, exiting...");
            return;
        }
       
        log.info("Finished!");
    }
}
