//  Copyright (C) University of Vienna, 2013-2015
//  All rights reserved.
//
//  Author: Ali Baharev
//
//  Please refer to the LICENSE.txt file in the top directory
//
package sdms;

import java.util.ArrayList;
import java.util.List;
import java.util.SortedSet;
import java.util.TreeMap;
import java.util.TreeSet;

public interface NLPProblem {
    
    /** In the under-determined situation, the least influential variables are artificially fixed */
    SubNLPProblem generateSubOptProblem(SortedSet<String>  selectedConstraintsWeightMap,
                                        TreeMap<String, Double>  fixedVariableValueMap,
                                        TreeSet<String>          varsToGuess,
                                        TreeMap<String,String[]> fullSparsityOnlyForDebugging);
    
    SubNLPProblem generateSubOptProblem(TreeSet<String> constraints,
                                        TreeSet<String> variables,
                                        TreeMap<String,String[]> fullSparsityOnlyForDebugging);
    
    double[] lowerBoundsOfVars(TreeSet<String> vars);
    
    double[] upperBoundsOfVars(TreeSet<String> vars);

    public int[] compareWithReferenceSolutions(List<Solution> allSolutions, double INFEASIBILITY_THRESHOLD, double maxDistance);
    
    ArrayList<TreeMap<String,Double>> getReferenceSolutions();
    
    int numberOfVars();
    
    TreeMap<String,Double> rowScales(SortedSet<String> cons);
    
    double[] columnScales(SortedSet<String> vars);
    
    TreeMap<String,Double> colScales(SortedSet<String> vars);
    
}
