//  Copyright (C) University of Vienna, 2013-2016
//  All rights reserved.
//
//  Author: Ali Baharev
//
//  Please refer to the LICENSE.txt file in the top directory
//
package sdms;

import java.util.ArrayList;
import java.util.Random;
import java.util.SortedSet;
import java.util.TreeMap;
import java.util.TreeSet;

import sdms.Violations;

public interface SubNLPProblem {
    
    double[] getLowerBoundsOfGuessedVars();
    
    double[] getUpperBoundsOfGuessedVars();
    
    /** returned in alphabetical order */
    double[] lowerBoundsOf(SortedSet<String> names);

    /** returned in alphabetical order */
    double[] upperBoundsOf(SortedSet<String> names);    
    
    int getNumberOfGuessedVars();
    
    int getNumberOfConstraints();

    TreeMap<String,Double> getLowerBounds(SortedSet<String> names);

    TreeMap<String,Double> getUpperBounds(SortedSet<String> names);
    
    TreeMap<String,Double> allLowerBounds();
    
    TreeMap<String,Double> allUpperBounds();
    
    void fixVars(TreeMap<String,Double> x);
    
    void setVarRoles(SortedSet<String> fixed, SortedSet<String> guessed);

    void setBounds(TreeMap<String,Double> lo, TreeMap<String,Double> up);
    
    /** [l, u] --> [l-c*(u-l), u+c*(u-l)] */
    void relaxBounds(SortedSet<String> names, double[] lb, double[] ub, double c);
    
    void dbgAllVarsCovered(SortedSet<String> x, SortedSet<String> y); // x and y are disjoint and their union contains all varnames 
    
    ArrayList<TreeMap<String,Double>> sampleUnderDeterminedWithQRP(Random rnd, 
                                                                   SortedSet<String> vars, 
                                                                   TreeMap<String,Double> rowScales, 
                                                                   TreeMap<String,Double> colScales, 
                                                                   int m, 
                                                                   int toFix);
    
    ArrayList<TreeMap<String,Double>> findLeastInfluentialWithQRP(TreeMap<String,Double> rowScales, 
                                                                  TreeMap<String,Double> columnScales, 
                                                                  TreeMap<String,Double> xPart, 
                                                                  TreeSet<String> yVars,
                                                                  int m, 
                                                                  TreeSet<String> yLast,
                                                                  int nToFix,
                                                                  Random rnd);
  
    Violations allViolations(TreeMap<String, Double> x, TreeMap<String, Double> rowScales, TreeMap<String, Double> colScales);
    
    TreeMap<String,Double> dbgResiduals(TreeMap<String,Double> x);
    
    @Override
    String toString();

    ArrayList<Solution> denseNewtonIterate(ArrayList<Solution> allPartialSolutions);
    
    void showConditionNumbers(ArrayList<Solution> allPartialSolutions);
}

