//  Copyright (C) University of Vienna, 2013-2016
//  All rights reserved.
//
//  Author: Ali Baharev
//
//  Please refer to the LICENSE.txt file in the top directory
//
package sdms.impl;

import lmbopt.fun.DagProblem;
import lmbopt.model.Lmbopt;
import problem.Problem;
import tools.console.DefaultConsole;
import tools.math.RealVector;
import tools.math.function.FunctionSolverOptions;
import tools.opt.SolverResult;
import tools.options.IllegalValueException;

public class Bug {
    
//    static {
//
//        ClassLoader.getSystemClassLoader().setDefaultAssertionStatus(true);
//        
//        String propertyName = "log4j.configurationFile";
//        
//        if (System.getProperty(propertyName)==null) {
//            
//            System.setProperty(propertyName, "config/Leak.xml");
//        }
//    }
    
    private static Lmbopt createLmbopt() {

        try {
            tools.options.Options options = Lmbopt.getGlobalOptions();
            
            //options.setValue(OptSolverOptions.OPTION_MAX_ITER_POSINT, 1000);
            options.setValue(Lmbopt.OPTION_MAX_GRAD_CALLS_POSINT, 1000);
            
           // options.setValue(OptSolverOptions.OPTION_ACCURACY_POSDOUBLE, 1.0e-12);
            options.setValue(Lmbopt.OPTION_GRAD_ACC_POSDOUBLE, 1.0e-12);
            
            options.setValue(Lmbopt.OPTION_SOL_CHECK_TOL_POSDOUBLE, 1.0e-12);
            //opts.setValue(Lmbopt.OPTION_OUTPUT_DIR_STRING, "/some/path");
            options.setValue(Lmbopt.OPTION_WRITE_OUTPUT_FILE, false);
            options.setValue(Lmbopt.OPTION_SOLUTION_WRITE_TESTENV_BOOLEAN, false);
            options.setValue(Lmbopt.OPTION_MEASURE_TIME_BOOLEAN, false);
            
            Lmbopt.setGlobalOptions(options);
        } 
        catch (IllegalValueException e) {
            e.printStackTrace();
            throw new AssertionError();
        }
        
        Lmbopt lmbopt=Lmbopt.getInstance();
        lmbopt.setConsole(new DefaultConsole()); 
        
        return lmbopt;
    }

    public static void main(String[] args) throws Throwable {
  
        int startPoint = 3; // valid indices are 0..3
        
//---------------------------------------------------------        
//        int testCase = 3;
//
//        String[] allCases = { "data/lmbopt/test01.mod", "data/lmbopt/test02.mod", "data/lmbopt/test03.mod" };
//        double[] allStarts= { 0.14727519928815158, 0.029690101831163496, 0.18121573805221447 };
//
//        // Convert from Arnold's one based index into zero based Java index
//        String file = allCases[testCase-1];
//        double start = allStarts[testCase-1];
//
//        RealVector startingPoint = new RealVector(1);
//        
//        startingPoint.set(0, start);
//---------------------------------------------------------         
        
        String file =  "data/lmbopt/test04.mod";
       
        System.out.println("Running with: "+file);
        
        Sdms.setProblemCheck(false, false, true, 1e-6);
        
        Problem prob = Sdms.readProblem(file, false);
        
        prob.getModel().setVarScaling(new RealVector(1,1e-3));

        DagProblem optimizationProblem = new DagProblem(prob, new DefaultConsole(), false);
        
        int x1 = optimizationProblem.getP().getModel().getVar("x1").getIndex();
        int V2 = optimizationProblem.getP().getModel().getVar("V2").getIndex();
        RealVector startingPoint = new RealVector(2);

        double[][] start = new double[][] { {0.7837058618286671, 2.209363477437332  }, 
                                            {0.5851468904697678, 3.7466956261476936 },
                                            {0.7563234276784557, 3.336043246352399  },
                                            {0.8361844546835389, 2.6524546687403383 } };
        
        startingPoint.set(x1, start[startPoint][0]);
        startingPoint.set(V2, start[startPoint][1]);        
        
        FunctionSolverOptions opts = new  FunctionSolverOptions(0, 1e-5, 300);
        
        System.out.println(optimizationProblem);
        
        System.out.println("x0="+startingPoint);
        
        SolverResult sr = Sdms.solveSubproblem(optimizationProblem, startingPoint, createLmbopt(), opts);
        
        System.out.println("Result: "+sr.getStatus());
        
        return;
    }
}
