//  Copyright (C) University of Vienna, 2013-2015
//  All rights reserved.
//
//  Author: Ali Baharev
//
//  Please refer to the LICENSE.txt file in the top directory
//
package sdms.impl;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import tools.opt.SolverResult;
import tools.opt.ProblemSolutionStatus;
import tools.math.RealVector;
import tools.math.function.FunctionSolver;
import tools.math.function.FunctionSolverOptions;

final class SolverBoundConstraint extends SolverBase {
    
    private static final Logger log = LogManager.getLogger();

    private FunctionSolver solver;
    private FunctionSolverOptions solverOptions;
    
    private static final boolean ERROR_OCCURED = true;
    private static final boolean SUCCESS       = false;

    SolverBoundConstraint(FunctionSolver solver, FunctionSolverOptions solverOptions) {
        this.solver = solver;
        this.solverOptions = solverOptions;
        // TODO Ask Feri to provide a toString() for FunctionSolverOptions
    }

    @Override
    protected SolutionInfo runSolver(SubNLPProblemImpl subProb, RealVector startingPoint) throws Exception {

        SolverResult ps = Sdms.solveSubproblem(subProb.getDagProblem(), startingPoint, solver, solverOptions);
        
        ProblemSolutionStatus status = ps.getStatus();

        if (status != ProblemSolutionStatus.ACCURACY_ACHIEVED) {

            log.debug("Solver returned: " + status);
            
            return new SolutionInfo(ps.getnObjEval(), ps.getnObjGradEval(), Double.NaN, null, ERROR_OCCURED);
        }
        else {
            
            return new SolutionInfo(ps.getnObjEval(), ps.getnObjGradEval(), ps.getBestF(), ps.getBest(), SUCCESS);
        }    
    }

    @Override
    protected String solverOptionsToString() {
        
        return solverOptions.toString();
    }
}
