//  Copyright (C) University of Vienna, 2013-2015
//  All rights reserved.
//
//  Author: Ali Baharev
//
//  Please refer to the LICENSE.txt file in the top directory
//
package sdms.impl;

import java.util.Random;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import sdms.StartingpointGenerator;

final class StartingpointGeneratorImpl implements StartingpointGenerator {
    
    private static final Logger log = LogManager.getLogger();
    
    private final Random rng;
    private double[] lb;
    private double[] ub;
    
    StartingpointGeneratorImpl() {
        rng = new Random(31L);
    }
    
    StartingpointGeneratorImpl(Random rnd) {
        rng = rnd;
    }

    @Override
    public boolean setNext() {

        return true;
    }

    @Override
    public void setVarBounds(double[] varLb, double[] varUb) {

        assert varLb.length > 0;
        assert varLb.length==varUb.length;
        
        for (int i=0; i<varLb.length; ++i) {
            double l = varLb[i];
            double u = varUb[i];
            if (l > u) {
                log.fatal("Invalid bounds, index "+i+", lb = "+l+", ub = "+u);
                throw new AssertionError();
            }
        }
        
        this.lb = varLb;
        this.ub = varUb;
    }

    @Override
    public double[] getNextGuess(double LB, double UB) {
        
        double[] x = new double[lb.length];
        
        for (int i=0; i<lb.length; ++i) {
            
            double l = (lb[i]!=Double.NEGATIVE_INFINITY) ? lb[i] : LB;
            
            double u = (ub[i]!=Double.POSITIVE_INFINITY) ? ub[i] : UB;
            
            x[i] = l + (u-l)*rng.nextDouble();
        }

        return x;
    }
    
    @Override
    public double[] getNextGuess() {
        
        double[] x = new double[lb.length];
        
        for (int i=0; i<lb.length; ++i) {
 
            x[i] = lb[i] + (ub[i]-lb[i])*rng.nextDouble();
        }

        return x;
    }
}
